/*
 * Attribute.cpp
 *
 * Attribute class methods
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#include "Attribute.h"

#include <iostream>
#include <sstream>
#include <cstdlib>
#include <cmath>

using namespace std;

Attribute::Attribute(){
	name = "";
	max = 1;
	min = 0;
	output = false;
}

Attribute::Attribute(const Attribute &orig){
	name = orig.name;
	max = orig.max;
	min = orig.min;
	values = orig.values;
	output = orig.output;
}

void Attribute::load(ifstream& fi, string& buf){
	getline(fi,name,' ');

	getline(fi,buf);
	istringstream iss(buf);
	if(buf.c_str()[0] == '{'){ //nominal
		getline(iss,buf,'{');
		getline(iss,buf,'}');
		values = split(buf);
	}
	else{ //number
		getline(iss,buf,'[');
		getline(iss,buf,']');
		values = split(buf);
		max = atof(trim(values[1]).c_str());
		min = atof(trim(values[0]).c_str());
		values.clear();
	}

}

float Attribute::getValue(string& s){
	float ret = -1;
	trim(s);
	if(values.size() == 0){
		ret = atof(s.c_str());
		ret = (ret - min)/(max-min);
	}
	else{
		for(int i = 0; i < values.size() && ret < 0; i++){
			if(s.compare(values[i]) == 0)
				ret = i;
		}
		ret = ret/((values.size()-1)*1.0);
	}

	return ret;
}

vector<string> Attribute::split(string &s){
	stringstream ss(s);
	vector<string> ret;

	string item;
	while (getline(ss, item, ',')){
		ret.push_back(trim(item));
	}
	return ret;
}

string Attribute::trim(string &s){
	s.erase(remove_if( s.begin(), s.end(), RemoveDelimiter()), s.end());
	return s;
}

