/*
 * Attribute.h
 *
 * Attribute class header
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#ifndef ATTRIBUTE_H_
#define ATTRIBUTE_H_

using namespace std;
#include <vector>
#include <string>
#include <fstream>
#include <algorithm>

/* Structure used to remove delimiters when reading files */
struct RemoveDelimiter
{
  bool operator()(char c)
  {
    return (c =='\r' || c =='\t' || c == ' ' || c == '\n');
  }
};

/* Attribute class */
class Attribute {
private:
	string name;			//Name of the attribute
	float max;				//Maximum value of numeric attributes
	float min;				//Minimum value of numeric attributes
	vector<string> values;	//Array of valid values for categorical attributes
	bool output;			//Output setting of the attribute

	/* Auxiliary function to split a string */
	vector<string> split(string &s);
	/* Auxiliary function to trim a string */
	string trim(string &s);

public:
	/* Default Constructor, creates a numeric attribute from 0 to 1 */
	Attribute();

	/* Copy Constructor */
	Attribute(const Attribute &orig);

	/* Function to read the header of an attribute from file
	 * ifstream fi: Input data
	 * string buf: Reading buffer
	 */
	void load(ifstream &fi, string& buf);

	/* Function to get the name of the attribute*/
	inline string getName(){ return name; }
	/* Function to get the minimum value of the attribute*/
	inline float getMin(){ return min; }
	/* Function to get the maximum value of the attribute*/
	inline float getMax(){ return max; }
	/* Function to get the list of values of the attribute*/
	inline vector<string> getValues(){ return values; }
	/* Function to get the output value of the attribute*/
	inline bool getOutput(){ return output; }
	/* Function to set the output value of the attribute*/
	inline void setOutput(bool v){ output = v; }

	/* Function to normalize the value of an input of the attribute*/
	float getValue(string& s);
};

#endif /* ATTRIBUTE_H_ */
