/*
 * CudaDistanceMatrix.h
 *
 * Distance kernel headers
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#ifndef CUDADISTANCEMATRIX_H_
#define CUDADISTANCEMATRIX_H_

#include "const.h"
#include <cstdio>

/* Computes a chunk of the distance matrix */
__global__ void distanceKernel(float* training, float* test, int nVariables, int trainingSize, int testSize, float* matrix, int chunkTrain, int nChunksTrain, int nNumeric, int selections);

/* Computes a chunk of the distance matrix using the Center kNN rule */
__global__ void distanceCenterKernel(float* training, float* test, int nVariables, int trainingSize, int testSize, float* matrix, int chunkTrain, int nChunksTrain, int nNumeric, int selections, int nOutput);

/* Computes a chunk of the distance matrix using skipping the training instances that belong to the same class than the test instance */
__global__ void distanceRadiusKernel(float* training, float* test, int nVariables, int trainingSize, int testSize, float* matrix, int chunkTrain, int nChunksTrain, int nNumeric);

/* Computes a chunk of the distance matrix using the kNN Adaptative rule */
__global__ void adaptiveDistanceKernel(float* training, float* test, int nVariables, int trainingSize, int testSize, float* matrix, int chunkTrain, int nChunksTrain, int nNumeric, int selections, float* radius);

/* Computes a chunk of the distance matrix using ignoring the distance between the instances that have the same index in both training and test sets*/
__global__ void distanceOneOutKernel(float* training, float* test, int nVariables, int trainingSize, int testSize, float* matrix, int chunkTrain, int nChunksTrain, int nNumeric, int selections, unsigned long int ofsetTrain, unsigned long int ofsetTest);

/* Computes a chunk of the distance matrix and the votes of the symmetric kNN rule*/
__global__ void distanceSymmetricKernel(float* training, float* test, int nVariables, int trainingSize, int testSize, float* matrix, int chunkTrain, int nChunksTrain, int nNumeric, int selections, int nOutput, float* distances, int* globalVotes, int valid);
#endif /* CUDADISTANCEMATRIX_H_ */
