/*
 * CudaSqrt.cu
 *
 * Square root kernel implementation
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#include "CudaSqrt.h"

__global__ void cuSqrt(float* valuesSource, unsigned long int* indexSource, float* valuesDestination, unsigned long int* indexDestination, int totalValues, int iterations){
	int tid = threadIdx.x;
	int bid = blockIdx.x;

	int index = tid;

	// Setting pointers to their positions in memory
	float* vs = &valuesSource[blockDim.x * iterations * bid];
	unsigned long int* is = &indexSource[blockDim.x * iterations * bid];
	float* vd = &valuesDestination[blockDim.x * iterations * bid];
	unsigned long int* id = &indexDestination[blockDim.x * iterations * bid];

	for(int i = 0; i < iterations; i++){
		if(index < totalValues){
			vd[index] = sqrt(vs[index]);	//SQRT operation on distances
			id[index] = is[index];			//Copying the index to the final structure
		}
		index += blockDim.x;
	}
}


__global__ void cuCopy(float* valuesSource, unsigned long int* indexSource, float* valuesDestination, unsigned long int* indexDestination, int totalValues, int iterations){
	int tid = threadIdx.x;
	int bid = blockIdx.x;

	int index = tid;
	// Setting pointers to their positions in memory
	float* vs = &valuesSource[blockDim.x * iterations * bid];
	unsigned long int* is = &indexSource[blockDim.x * iterations * bid];
	float* vd = &valuesDestination[blockDim.x * iterations * bid];
	unsigned long int* id = &indexDestination[blockDim.x * iterations * bid];

	for(int i = 0; i < iterations; i++){
		if(index < totalValues){
			vd[index] = vs[index];	//Copying the distance value to the final structure
			id[index] = is[index];	//Copying the index to the final structure
		}
		index += blockDim.x;
	}
}


__global__ void cuGetLast(float* valuesSource, unsigned long int* indexSource, float* valuesDestination, unsigned long int* indexDestination, int totalValues, int k){
	int tid = threadIdx.x;
	int bid = blockIdx.x;

	// Setting pointers to their positions in memory
	int tot = totalValues - (blockDim.x * bid);
	int index = (blockDim.x * bid + tid);
	unsigned long int* is = &indexSource[index*k];
	float* vs = &valuesSource[index*k];

	if(tid < tot){
		indexDestination[index] = is[k-1]; //Copying the index of the farthest distance to the final structure
		valuesDestination[index] = vs[k-1]; //Copying the farthest distance value to the final structure
		//the sqrt is no performed because the comparison will be done during the distance matrix computation
	}
}
