/*
 * CudaSqrt.h
 *
 * Square root operation kernel headers
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#ifndef CUDASQRT_H_
#define CUDASQRT_H_

#include "const.h"
#include <cstdio>

/* Performs the SQRT operation on the neighborhood distances and copy the index values to their final structures */
__global__ void cuSqrt(float* valuesSource, unsigned long int* indexSource, float* valuesDestination, unsigned long int* indexDestination, int totalValues, int iterations);

/* Copies the neighborhood distances and the index values to their final structures */
__global__ void cuCopy(float* valuesSource, unsigned long int* indexSource, float* valuesDestination, unsigned long int* indexDestination, int totalValues, int iterations);

/* Copies the farthest neighbor distance and index values to their final structures for the symmetric kNN rule */
__global__ void cuGetLast(float* valuesSource, unsigned long int* indexSource, float* valuesDestination, unsigned long int* indexDestination, int totalValues, int k);
#endif /* CUDASQRT_H_ */
