/*
 * Dataset.cpp
 *
 * Dataset class method file
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#include "Dataset.h"
#include <iostream>

using namespace std;

Dataset::Dataset() {
	nInst = 0;
	nVariables = 0;
	variables = 0;
	nOutput = 0;
	nNumeric = 0;
}

Dataset::Dataset(const Dataset& orig){
	nInst = orig.nInst;
	nVariables = orig.nVariables;
	variables = orig.variables;
	nOutput = orig.nOutput;
	name = orig.name;
	attributes = orig.attributes;
	nNumeric = orig.nNumeric;
}

Dataset::~Dataset() {
	if(variables != 0)
		delete[] variables;
	variables = 0;
}

void Dataset::allocate(){
	variables = new float[nInst*nVariables];
}

void Dataset::load(char* filename){
	std::ifstream fi(filename);

	if(fi.is_open()){
		int nlines = 0;
		//processing input header
		std::string buf;

		//Get dataset name
		std::getline(fi,buf,' ');
		if(buf.compare("@relation")==0)
			std::getline(fi,name);
		else{
			std::cerr<<"Error reading file "<<filename<<std::endl<<"@relation token not found"<<std::endl;
			exit(-1);
		}
		nlines++;

		//Get dataset attributes
		std::getline(fi,buf,' ');
		while(buf.compare("@attribute")==0){
			nlines++;
			Attribute a;
			a.load(fi, buf);
			attributes.push_back(a);

			std::getline(fi,buf,' ');
		}

		//Skip number of inputs
		std::getline(fi,buf);
		nlines++;

		//Set outputs
		std::getline(fi,buf,' '); //read @outputs
		std::getline(fi,buf); //read the output name
		buf.erase(remove_if( buf.begin(), buf.end(), RemoveDelimiter()), buf.end());
		nlines++;

		bool error = true;
		for(int i = 0; i < attributes.size() && error; i++){
			if(buf.compare(attributes[i].getName()) == 0){
				error = false;
				attributes[i].setOutput(true);
				nOutput = attributes[i].getValues().size();
			}
		}

		if(error){
			std::cerr<<"Error reading outputs"<<std::endl;
			exit(-1);
		}

		//Skip @data tag
		std::getline(fi,buf); //read @data
		nlines++;

		std::ifstream count(filename);

		//Geting the number of instances
		nInst = 0;
		while(std::getline(count, buf))
			nInst++;

		nInst -= nlines; //@relation @inputs @outputs @data
		nVariables = attributes.size();

		count.close();

		//count numeric attributes
		nNumeric = 0;
		for(int i = 0; i < nVariables; i++)
			if(!isNominal(i))
				nNumeric++;

		//allocating memory for the instances
		allocate();


		//The dataset is stored in a transposed way
		for(int i = 0; i < nInst; i++){
			int numeric = 0;
			int nominal = 0;
			for(int j = 0; j < nVariables; j++){
				if(j == nVariables -1)
					std::getline(fi,buf);
				else
					std::getline(fi,buf,',');

				//reordering the attributes to have all numeric attributes at the beginning and all non numeric at the end
				if(isNominal(j)){
					variables[(nNumeric+nominal) * nInst + i] = attributes[j].getValue(buf);
					nominal++;
				}
				else{
					variables[numeric * nInst + i] = attributes[j].getValue(buf);
					numeric++;
				}
			}
		}
	}

	fi.close();
}

void Dataset::print(){
       for(int i = 0; i < nInst; i++){
               for(int j = 0; j < nVariables; j++){
                       cerr<<variables[j * nInst + i]<<',';
               }
               cerr<<endl;
       }
}

