/*
 * Dataset.h
 *
 * Dataset class header
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#ifndef DATASET_H_
#define DATASET_H_

#include <string>
#include <vector>
#include "Attribute.h"

using namespace std;

/* Dataset class */
class Dataset {
protected:
	int nVariables;		//Number of variables of the dataset
	int nInst;			//Number of instances of the dataset
	float* variables;	//List of normalized values of the dataset
	int nOutput;		//Number of output values
	int nNumeric;		//Number of numeric values

	string name;					//Name of the dataset
	vector<Attribute> attributes;	//List of attributes of the dataset

public:
	/* Default constructor, creates an empty dataset*/
	Dataset();
	/* Copy constructor*/
	Dataset(const Dataset& orig);
	/* Destructor*/
	virtual ~Dataset();

	/* Function to get the number of variables of the dataset*/
	inline int getNVariables() { return nVariables; }
	/* Function to set the number of variables of the dataset*/
	inline void setNVariables(int n) { nVariables = n; }
	/* Function to get the number of instances of the dataset*/
	inline int getNInst() { return nInst; }
	/* Function to set the number of instances of the dataset*/
	inline void setNInst(int n) { nInst = n; }
	/* Function to get the number of output values of the dataset*/
	inline int getNOutput() { return nOutput; }
	/* Function to get the number of output value for a specific index*/
	inline float getOutputValue( int i) { return i/((nOutput-1)*1.0); }
	/* Function to get the name of the dataset*/
	inline string getName() { return name; }
	/* Function to get the value of the attribute att at the instance inst of the dataset*/
	inline float getValue(int inst, int att) { return variables[att*nInst+inst]; }
	/* Function to get the class of an instance i of the dataset*/
	inline float getClass(int i) { return getValue(i, nVariables-1); }
	/* Function to get the class number of an instance i of the dataset*/
	inline int getClassNumber(int i) { return getValue(i, nVariables-1)*(attributes[nVariables-1].getValues().size()-1); }
	/* Function to identifiy nominal attributes of the dataset*/
	inline bool isNominal(int c) { return attributes[c].getValues().size() != 0; }
	/* Function to get the number of numeric variables of the dataset*/
	inline int getNNumeric() { return nNumeric; }

	/* Function to load the dataset from a file
	 * Filename is the path to a Keel Format dataset
	 */
	void load(char* filename);

	/* Function to allocate space for the dataset*/
	virtual void allocate();

	/* Function to print the dataset*/
	void print();
};

#endif /* DATASET_H_ */
