/*
 * DatasetGPU.cpp
 *
 * DatasetGPU class methods file
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#include "DatasetGPU.h"
#include "util.h"

DatasetGPU::DatasetGPU() : Dataset() {
	variablesGPU = 0;
	e=0;
	copyStream=0;
}

DatasetGPU::DatasetGPU(const DatasetGPU& orig){
	nInst = orig.nInst;
	nVariables = orig.nVariables;
	nOutput = orig.nOutput;
	name = orig.name;
	attributes = orig.attributes;
	nNumeric = orig.nNumeric;

	if(orig.variables != 0){
		allocate();
		cudaMemcpy(variables,orig.variables,sizeof(float)*nVariables*nInst,cudaMemcpyHostToHost);
	}
	else{
		variables = 0;
		variablesGPU = 0;
		e = 0;
		copyStream = 0;
	}
}

DatasetGPU::~DatasetGPU(){
	if(variablesGPU != 0){
		freeGPU();
		cudaFreeHost(variables);
		variables=0;
	}
}

void DatasetGPU::allocate(){
	cudaMallocHost(&variables, sizeof(float)*(nInst*nVariables));
	allocateGPU();
}

void DatasetGPU::allocateGPU(){
	unsigned long int size = TRAININGCHUNK > TESTCHUNK ? sizeof(float)*TRAININGCHUNK*nVariables : sizeof(float)*TESTCHUNK*nVariables;
	cudaMalloc(&variablesGPU,size );

	cudaEventCreate(&e);
	cudaStreamCreate(&copyStream);
}

void DatasetGPU::copyChunk(int i, int n){
	if(i < n){
		int size = (nInst - TRAININGCHUNK*i) < TRAININGCHUNK ? nInst - TRAININGCHUNK*i : TRAININGCHUNK;
		for(int k = 0; k < nVariables; k++)
			cudaMemcpyAsync(variablesGPU+(TRAININGCHUNK*k), variables+(nInst*k + TRAININGCHUNK*i), sizeof(float)*size,cudaMemcpyHostToDevice,copyStream);
	}
	cudaEventRecord(e,copyStream);
}

void DatasetGPU::copyChunkTest(int i, int n){
	if(i < n){
		int size = (nInst - TESTCHUNK*i) < TESTCHUNK ? nInst - TESTCHUNK*i : TESTCHUNK;
		for(int k = 0; k < nVariables; k++)
			cudaMemcpyAsync(variablesGPU+(TESTCHUNK*k), variables+(nInst*k + TESTCHUNK*i), sizeof(float)*size,cudaMemcpyHostToDevice,copyStream);
	}
	cudaEventRecord(e,copyStream);
}

void DatasetGPU::freeGPU(){
	cudaFree(variablesGPU);
	variablesGPU = 0;

	cudaEventDestroy(e);
	cudaStreamDestroy(copyStream);
}
