/*
 * DatasetGPU.h
 *
 * DatasetGPU class header
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#ifndef DATASETGPU_H_
#define DATASETGPU_H_

#include <cuda_runtime.h>
#include <iostream>
#include "Dataset.h"
#include "const.h"

using namespace std;

/* DatasetGPU class inherits from Dataset class adding GPU structures and allocations */
class DatasetGPU : public Dataset{
protected:
	float* variablesGPU;		//GPU array for variables
	cudaEvent_t e;				//CUDA event that controls asyncronous copies for this dataset
	cudaStream_t copyStream;	//CUDA stream where asyncronous copies are done for this dataset
public:
	/* Default constructor */
	DatasetGPU();
	/* Copy constructor*/
	DatasetGPU(const DatasetGPU& orig);
	/*Destructor*/
	virtual ~DatasetGPU();

	/* CPU data allocation function*/
	virtual void allocate();
	/* GPU data allocation function*/
	virtual void allocateGPU();
	/* Copy chunk function for training set*/
	virtual void copyChunk(int i, int n);
	/* Copy chunk function for test set*/
	virtual void copyChunkTest(int i, int n);
	/* Free GPU data*/
	virtual void freeGPU();

	/* Gets the GPU pointer to the array of data, used to call kernel functions */
	inline float* getDevicePointer() { return variablesGPU; }
	/* Waits for a memory copy to finish*/
	inline void sync() { cudaEventSynchronize(e); }
};

#endif /* DATASETGPU_H_ */
