/*
 * KNNAdaptiveGPU.cpp
 *
 * KNNAdaptiveGPU class methods file
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#include "KNNAdaptiveGPU.h"

KNNAdaptiveGPU::KNNAdaptiveGPU(DatasetGPU* train, DatasetGPU* tst) : KnnGPU(train,tst) {
}

KNNAdaptiveGPU::~KNNAdaptiveGPU() {
}

void KNNAdaptiveGPU::ComputeDistanceGPU(int trainChunk, int testChunk){
	DatasetAdaptive* tr = (DatasetAdaptive*)training;
	int nthreads = THREADSMULT;
	int block = TESTCHUNK;
	int shared = training->getNVariables()*sizeof(float);

	adaptiveDistanceKernel<<<block, nthreads, shared, kernelStream>>>(deviceTraining->getDevicePointer(), deviceTest->getDevicePointer(), training->getNVariables(), trainingSize, testSize, deviceDistanceMatrix, trainChunk, trainingChunks, training->getNNumeric(), k, tr->getRadiusPointer());
}

void KNNAdaptiveGPU::ComputeSqrtGPU(){
	int threads = 128;
	int iterations = k;
	int totalValues = TESTCHUNK*k;
	int blocks = totalValues/(threads*iterations)+(totalValues%(threads*iterations) != 0);

	cuCopy<<<blocks,threads,0,kernelStream>>>(deviceNeighborValuesFirstStep, deviceNeighborIndexFirstStep, deviceNeighborValues, deviceNeighborIndex, totalValues, iterations);
}


void KNNAdaptiveGPU::initialize(){
	/* Computing radius value*/
	DatasetAdaptive* tr = (DatasetAdaptive*)training;
	tr->computeRadius();
}
