/*
 * KNNSymmetricGPU.h
 *
 * KNNSymmetricGPU class header
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#ifndef KNNSYMMETRIC_H_
#define KNNSYMMETRIC_H_

#include "KnnGPU.h"
#include "DatasetSymmetric.h"

/* KNNSymmetricGPU class inherits from KnnGPU class adding overriding some functions to use the farthest distance within the training set in the computations
 * and allowing the symmatric vote
 */
class KNNSymmetricGPU: public KnnGPU {
protected:
	int* globalVotes;			// CPU structure for the votes
	int* deviceGlobalVotes;	// GPU structure to compute the votes
	int* deviceGlobalVotesCopy;// GPU structure to copy the votes from GPU to CPU memory

	/* Copies the neigborhood to the CPU, including the computed symmetric votes */
	virtual void copyNeighbors(unsigned long int j);
public:
	/*Constructor*/
	KNNSymmetricGPU(DatasetGPU* train, DatasetGPU* tst);
	/*Destructor*/
	virtual ~KNNSymmetricGPU();

	/* Memory allocation for voting structures */
	void allocateGlobalVotes();
	/* Memory release of voting structures */
	void freeGlobalVotes();

	/* Initializes the algorithm computing the farthest distance for each training instance within the training set*/
	virtual void initialize();

	/* Wrapper for the modified distance version that computes the symmetric votes */
	virtual void ComputeDistanceGPU(int trainChunk, int testChunk);
	/* Wrapper for the SQRT kernel the copy of the voting structure is done here*/
	virtual void ComputeSqrtGPU();
	/* Modified version of the accuracy function to take into account the symmetric votes */
	virtual float computeAcuracy();
};

#endif /* KNNSYMMETRIC_H_ */
