/*
 * Knn.h
 *
 * Knn class header
 *
 *      Author: Pablo David Gutiérrez
 *      Author: Miguel Lastra
 *      Author: Jaume Barcardit
 *      Author: José Manuel Benítez
 *      Author: Francisco Herrera
 */

#ifndef KNN_H_
#define KNN_H_
#include <vector>
#include "Dataset.h"

/* kNN Class*/
class Knn {
protected:
	Dataset* training;		//Training set pointer
	Dataset* test;			//Test set pointer
	float* distanceMatrix;	//Distance array structure
	static int k;			//Number of neighbors (k)
	int trainingSize;		//Training size
	int testSize;			//Test size

	/* Computes the distance between two instances */
	virtual float distance(unsigned long int trainIndex, unsigned long int testIndex);
	/* Computes the selection of the neighborhood with an insertion scheme */
	virtual vector<unsigned long int> selection(unsigned long int inst);
	/* Computes the selection of the neighborhood with a quicksort scheme */
	virtual vector<unsigned long int> quick(vector<unsigned long int> index, int acum);
	/* Computes the median value of the supplied elements */
	unsigned long int median(unsigned long int a, unsigned long int b, unsigned long int c);

public:
	/* Default constructor, creates an empty algorithm*/
	Knn();
	/* Constructor with traingn and test set as parameters */
	Knn(Dataset* train, Dataset* tst);
	/* Destructor */
	virtual ~Knn();

	/* Sets the value of k */
	inline static void setK(int i) { k = i; }
	/* Gets the value of k */
	inline int static getK() { return k; }

	/* Computes the distance matrix between training and test set*/
	virtual void computeDistanceMatrix();

	/* Selects the neighborhood of each test instance and classifies it with the current distance matrix
	 * Returns the accuracy value
	 * */
	virtual float computeAcuracy();

	/* If required, computes information before the kNN computation*/
	virtual void initialize();
};

#endif /* KNN_H_ */
